import pandas as pd
from ovretl.loads_utils.calculate_single_load_total_quantities import (
    calculate_single_load_total_quantities,
    calculate_taxable_weight,
    calculate_weight_measurable,
)


def calculate_entity_loads(loads_df: pd.DataFrame, key: str):
    entity_loads = loads_df[~loads_df[key].isnull()]
    entity_loads = (
        entity_loads.groupby(key)
        .agg(
            {
                "total_number": "sum",
                "total_volume_in_cbm": "sum",
                "total_weight_in_kg": "sum",
                "hazardous": lambda load_hazardous: load_hazardous.any(),
                "lithium": lambda load_lithium: load_lithium.any(),
                "refrigerated": lambda load_refrigerated: load_refrigerated.any(),
                "magnetic": lambda load_magnetic: load_magnetic.any(),
                "non_stackable": lambda load_non_stackable: load_non_stackable.any(),
            }
        )
        .reset_index().drop_duplicates(subset=[key])
    )
    entity_loads["weight_measurable"] = entity_loads.apply(
        lambda load: calculate_weight_measurable(load["total_weight_in_kg"], load["total_volume_in_cbm"]), axis=1,
    )
    entity_loads["taxable_weight_in_kg"] = entity_loads.apply(
        lambda load: calculate_taxable_weight(load["total_weight_in_kg"], load["total_volume_in_cbm"]), axis=1,
    )
    return entity_loads


def calculate_propositions_loads(loads_df: pd.DataFrame):
    loads_df = loads_df.apply(calculate_single_load_total_quantities, axis=1)
    loads_df = loads_df.drop(columns=["total_volume_in_cbcm"])
    loads_df = loads_df.dropna(subset=["total_volume_in_cbm", "total_weight_in_kg"])
    propositions_loads = calculate_entity_loads(loads_df=loads_df, key="proposition_id")
    return propositions_loads


def calculate_shipments_propositions_loads(loads_df: pd.DataFrame):
    loads_df = loads_df.apply(calculate_single_load_total_quantities, axis=1)
    loads_df = loads_df.drop(columns=["total_volume_in_cbcm"])
    loads_df = loads_df.dropna(subset=["total_volume_in_cbm", "total_weight_in_kg"])
    propositions_loads = calculate_entity_loads(loads_df=loads_df, key="proposition_id")
    shipments_loads = calculate_entity_loads(loads_df=loads_df, key="shipment_id")
    return pd.concat([propositions_loads, shipments_loads], sort=False)



import pandas as pd

from ovretl import determine_import_status


def test_determine_import_status():
    shipment_truck = pd.Series(data={"freight_method": "truck"})
    shipment_import = pd.Series(
        data={
            "freight_method": "ocean",
            "pickup_country": "CN",
            "pickup_harbor_country": "CN",
            "delivery_country": "FR",
            "delivery_harbor_country": "BE",
        }
    )
    shipment_export = pd.Series(
        data={
            "freight_method": "ocean",
            "pickup_country": "BE",
            "pickup_harbor_country": "FR",
            "delivery_country": "US",
            "delivery_harbor_country": "US",
        }
    )
    shipment_cross_trade = pd.Series(
        data={
            "freight_method": "ocean",
            "pickup_country": "HK",
            "pickup_harbor_country": "HK",
            "delivery_country": "US",
            "delivery_harbor_country": "US",
        }
    )
    assert determine_import_status(shipment_truck) == "inland"
    assert determine_import_status(shipment_import) == "import"
    assert determine_import_status(shipment_export) == "export"
    assert determine_import_status(shipment_cross_trade) == "cross_trade"



import pandas as pd

KRONOS_ID = 125


def calculate_number_purchase_readied(activities_df: pd.DataFrame, shipments_df: pd.DataFrame):
    activities_purchase_ready_df = activities_df[
        (activities_df["header"] == "quotation_purchase_ready") & (activities_df["employee_id"] != KRONOS_ID)
    ]
    activities_purchase_ready_df = activities_purchase_ready_df.groupby("shipment_id").size()
    shipments_with_requotes = pd.merge(
        left=shipments_df,
        right=activities_purchase_ready_df.to_frame(name="nb_purchase_readied"),
        left_on="shipment_id",
        right_index=True,
        how="left",
    )
    shipments_with_requotes.loc[:, "nb_purchase_readied"] = shipments_with_requotes["nb_purchase_readied"].fillna(0)
    return shipments_with_requotes



import pandas as pd
import datetime
from dateutil.relativedelta import relativedelta
from pytz import utc


def get_week_number_from_jesus(date):
    return date.isocalendar()[1] + date.year * 52


def check_shipment_active_at_date(date, shipment: pd.Series):
    if shipment["cancelled"] == True:
        return False
    if shipment["shipment_status"] in [
        "new",
        "not_answered",
        "propositions_sent",
        "purchase_ready",
        "to_be_requoted",
        "not_answered_declined",
    ]:
        return False
    if pd.isna(shipment["operations_owner_id"]):
        return False
    if (pd.to_datetime(shipment["created_at"]) - date).days > 7:
        return False
    if pd.to_datetime(shipment["finished_date"]) < date:
        return False
    last_date = shipment["delivery_date"] if not pd.isna(shipment["delivery_date"]) else shipment["arrival_date"]
    if date > last_date:
        return False
    if (
        pd.isna(shipment["pickup_date"])
        and pd.isna(shipment["departure_date"])
        and pd.isna(shipment["arrival_date"])
        and pd.isna(shipment["delivery_date"])
    ):
        return True
    if not pd.isna(shipment["departure_date"]) and get_week_number_from_jesus(date) <= get_week_number_from_jesus(
        pd.to_datetime(shipment["departure_date"]) + relativedelta(days=2)
    ):
        return True

    if not pd.isna(shipment["arrival_date"]) and get_week_number_from_jesus(date) >= get_week_number_from_jesus(
        pd.to_datetime(shipment["arrival_date"]) + relativedelta(days=-10)
    ):
        return True
    return False


def determine_active_shipments_at_date(date, shipments_df: pd.DataFrame):
    shipments_df.loc[:, "active"] = shipments_df.apply(lambda s: check_shipment_active_at_date(date, s), axis=1)
    active_shipments = shipments_df[shipments_df["active"]]
    active_shipments.loc[:, "active_date"] = date
    return active_shipments


def determine_active_shipments(shipments_df: pd.DataFrame, date_end=datetime.datetime.now()):
    dates = pd.date_range(start="2020-01-01", end=date_end, freq="W-MON", tz=utc).to_series()
    df_list = []
    for date in dates:
        df_list.append(determine_active_shipments_at_date(date, shipments_df))
    active_shipments_data_df = pd.concat(df_list, sort=False)
    active_shipments_data_df = active_shipments_data_df[["active_date", "foresea_name", "operations",]]
    return active_shipments_data_df



import pandas as pd


def merge_with_propositions_containers(
    propositions_df: pd.DataFrame, propositions_containers_df: pd.DataFrame,) -> pd.DataFrame:
    propositions_df = pd.merge(
        propositions_df,
        propositions_containers_df[["teus", "tc", "hazardous", "refrigerated", "proposition_id"]],
        how="left",
        on="proposition_id",
    ).reset_index(drop=True).rename(columns={"hazardous": "containers_hazardous",
                                             "refrigerated": "containers_refrigerated"})
    return propositions_df


def merge_propositions_with_containers(
    propositions_df: pd.DataFrame, propositions_containers_df: pd.DataFrame, drop_duplicate_key="proposition_id",
):
    propositions_df = merge_with_propositions_containers(
        propositions_df=propositions_df,
        propositions_containers_df=propositions_containers_df
    )
    propositions_df = propositions_df.sort_values(by=["teus", "tc"], ascending=False)
    propositions_df = propositions_df[
        (~propositions_df.duplicated(subset=[drop_duplicate_key])) | (propositions_df[drop_duplicate_key].isnull())
    ]  # drop duplicates without dropping nan
    return propositions_df



import pandas as pd


def kronos_propositions_missing_categories(kronos_propositions_df: pd.DataFrame):
    category_complete = kronos_propositions_df.groupby("category").apply(
        lambda group: group["kronos_selected"].any()  # or group["selected"].any()
    )
    if False in category_complete.values:
        return category_complete.value_counts()[False]
    return 0


def add_missing_categories(kronos_propositions_with_ocp_df: pd.DataFrame):
    cq_fullness = kronos_propositions_with_ocp_df.groupby(by=["kronos_proposition_id"]).apply(
        kronos_propositions_missing_categories
    )

    kronos_propositions_with_ocp_df = pd.merge(
        kronos_propositions_with_ocp_df,
        cq_fullness.to_frame(name="missing_categories"),
        left_on="kronos_proposition_id",
        right_index=True,
    )
    kronos_propositions_with_ocp_df["full"] = kronos_propositions_with_ocp_df["missing_categories"].apply(
        lambda x: x == 0
    )
    kronos_propositions_with_ocp_df.loc[:, "proposition_id"] = kronos_propositions_with_ocp_df["proposition_id"].apply(
        lambda x: int(x) if not pd.isna(x) else x
    )
    return kronos_propositions_with_ocp_df
